import random

from geopandas import GeoDataFrame, GeoSeries, sjoin
from shapely.geometry import Point, LineString, Polygon
import numpy as np


class Bench:

    param_names = ['op']
    params = [('intersects', 'contains', 'within')]

    def setup(self, *args):
        triangles = GeoSeries(
            [Polygon([(random.random(), random.random()) for _ in range(3)])
             for _ in range(1000)])

        points = GeoSeries(
            [Point(x, y) for x, y in zip(np.random.random(10000),
                                         np.random.random(10000))])

        df1 = GeoDataFrame({'val1': np.random.randn(len(triangles)),
                            'geometry': triangles})
        df2 = GeoDataFrame({'val1': np.random.randn(len(points)),
                            'geometry': points})

        self.df1, self.df2 = df1, df2

    def time_sjoin(self, predicate):
        sjoin(self.df1, self.df2, predicate=predicate)



import random

import numpy as np
from geopandas import GeoSeries
from shapely.geometry import Point, Polygon, MultiPolygon


def with_attributes(**attrs):
    def decorator(func):
        for key, value in attrs.items():
            setattr(func, key, value)
        return func
    return decorator


class Bench:

    def setup(self, *args):
        self.points = GeoSeries([Point(i, i) for i in range(100000)])

        triangles = GeoSeries([Polygon([(random.random(), random.random())
                                        for _ in range(3)])
                               for _ in range(1000)])
        triangles2 = triangles.copy().iloc[np.random.choice(1000, 1000)]
        triangles3 = GeoSeries([Polygon([(random.random(), random.random())
                                         for _ in range(3)])
                                for _ in range(10000)])
        triangles4 = GeoSeries([
            MultiPolygon([
                Polygon([(random.random(), random.random()) for _ in range(3)])
            ]) for _ in range(10000)])
        triangle = Polygon([(random.random(), random.random())
                            for _ in range(3)])
        self.triangles, self.triangles2 = triangles, triangles2
        self.triangles_big = triangles3
        self.multi_triangles = triangles4
        self.triangle = triangle

    @with_attributes(param_names=['op'],
                     params=[('contains', 'crosses', 'disjoint', 'intersects',
                              'overlaps', 'touches', 'within', 'geom_equals',
                              'geom_almost_equals', 'geom_equals_exact')])
    def time_binary_predicate(self, op):
        getattr(self.triangles, op)(self.triangle)

    @with_attributes(param_names=['op'],
                     params=[('contains', 'crosses', 'disjoint', 'intersects',
                              'overlaps', 'touches', 'within', 'geom_equals',
                              'geom_almost_equals')])  # 'geom_equals_exact')])
    def time_binary_predicate_vector(self, op):
        getattr(self.triangles, op)(self.triangles2)

    @with_attributes(param_names=['op'],
                     params=[('distance')])
    def time_binary_float(self, op):
        getattr(self.triangles, op)(self.triangle)

    @with_attributes(param_names=['op'],
                     params=[('distance')])
    def time_binary_float_vector(self, op):
        getattr(self.triangles, op)(self.triangles2)

    @with_attributes(param_names=['op'],
                     params=[('difference', 'symmetric_difference', 'union',
                              'intersection')])
    def time_binary_geo(self, op):
        getattr(self.triangles, op)(self.triangle)

    @with_attributes(param_names=['op'],
                     params=[('difference', 'symmetric_difference', 'union',
                              'intersection')])
    def time_binary_geo_vector(self, op):
        getattr(self.triangles, op)(self.triangles2)

    @with_attributes(param_names=['op'],
                     params=[('is_valid', 'is_empty', 'is_simple', 'is_ring')])
    def time_unary_predicate(self, op):
        getattr(self.triangles, op)

    @with_attributes(param_names=['op'],
                     params=[('area', 'length')])
    def time_unary_float(self, op):
        getattr(self.triangles_big, op)

    @with_attributes(param_names=['op'],
                     params=[('boundary', 'centroid', 'convex_hull',
                              'envelope', 'exterior', 'interiors')])
    def time_unary_geo(self, op):
        getattr(self.triangles, op)

    def time_unary_geo_representative_point(self, *args):
        getattr(self.triangles, 'representative_point')()

    def time_geom_type(self, *args):
        self.triangles_big.geom_type

    def time_bounds(self, *args):
        self.triangles.bounds

    def time_unary_union(self, *args):
        self.triangles.unary_union

    def time_buffer(self, *args):
        self.points.buffer(2)

    def time_explode(self, *args):
        self.multi_triangles.explode()


# TODO
# project, interpolate, affine_transform, translate, rotate, scale, skew
# cx indexer



"""
See generate_legacy_storage_files.py for the creation of the legacy files.

"""
from contextlib import contextmanager
from packaging.version import Version
import glob
import os
import pathlib

import pandas as pd

import pyproj

import pytest
from geopandas.testing import assert_geodataframe_equal
from geopandas import _compat as compat
import geopandas
from shapely.geometry import Point

DATA_PATH = pathlib.Path(os.path.dirname(__file__)) / "data"


@pytest.fixture(scope="module")
def current_pickle_data():
    # our current version pickle data
    from .generate_legacy_storage_files import create_pickle_data

    return create_pickle_data()


files = glob.glob(str(DATA_PATH / "pickle" / "*.pickle"))


@pytest.fixture(params=files, ids=[p.split("/")[-1] for p in files])
def legacy_pickle(request):
    return request.param


@contextmanager
def with_use_pygeos(option):
    orig = geopandas.options.use_pygeos
    geopandas.options.use_pygeos = option
    try:
        yield
    finally:
        geopandas.options.use_pygeos = orig


@pytest.mark.skipif(
    compat.USE_SHAPELY_20
    or compat.USE_PYGEOS
    or (Version(pyproj.__version__) < Version("2.4")),
    reason=(
        "shapely 2.0/pygeos-based unpickling currently only works for "
        "shapely-2.0/pygeos-written files; "
        "old pyproj versions can't read pickles from newer pyproj versions"
    ),
)
def test_legacy_pickles(current_pickle_data, legacy_pickle):
    result = pd.read_pickle(legacy_pickle)

    for name, value in result.items():
        expected = current_pickle_data[name]
        assert_geodataframe_equal(value, expected)


def test_round_trip_current(tmpdir, current_pickle_data):
    data = current_pickle_data

    for name, value in data.items():
        path = str(tmpdir / "{}.pickle".format(name))
        value.to_pickle(path)
        result = pd.read_pickle(path)
        assert_geodataframe_equal(result, value)
        assert isinstance(result.has_sindex, bool)


def _create_gdf():
    return geopandas.GeoDataFrame(
        {"a": [0.1, 0.2, 0.3], "geometry": [Point(1, 1), Point(2, 2), Point(3, 3)]},
        crs="EPSG:4326",
    )


@pytest.mark.skipif(not compat.HAS_PYGEOS, reason="requires pygeos to test #1745")
def test_pygeos_switch(tmpdir):
    # writing and reading with pygeos disabled
    with with_use_pygeos(False):
        gdf = _create_gdf()
        path = str(tmpdir / "gdf_crs1.pickle")
        gdf.to_pickle(path)
        result = pd.read_pickle(path)
        assert_geodataframe_equal(result, gdf)

    # writing without pygeos, reading with pygeos
    with with_use_pygeos(False):
        gdf = _create_gdf()
        path = str(tmpdir / "gdf_crs1.pickle")
        gdf.to_pickle(path)

    with with_use_pygeos(True):
        result = pd.read_pickle(path)
        gdf = _create_gdf()
        assert_geodataframe_equal(result, gdf)

    # writing with pygeos, reading without pygeos
    with with_use_pygeos(True):
        gdf = _create_gdf()
        path = str(tmpdir / "gdf_crs1.pickle")
        gdf.to_pickle(path)

    with with_use_pygeos(False):
        result = pd.read_pickle(path)
        gdf = _create_gdf()
        assert_geodataframe_equal(result, gdf)



"""
Visualizing NYC Boroughs
------------------------

Visualize the Boroughs of New York City with Geopandas.

This example generates many images that are used in the documentation. See
the `Geometric Manipulations <geometric_manipulations>` example for more
details.

First we'll import a dataset containing each borough in New York City. We'll
use the ``datasets`` module to handle this quickly.
"""
import numpy as np
import matplotlib.pyplot as plt
from shapely.geometry import Point
from geopandas import GeoSeries, GeoDataFrame
import geopandas as gpd

np.random.seed(1)
DPI = 100

path_nybb = gpd.datasets.get_path('nybb')
boros = GeoDataFrame.from_file(path_nybb)
boros = boros.set_index('BoroCode')
boros

##############################################################################
# Next, we'll plot the raw data
ax = boros.plot()
plt.xticks(rotation=90)
plt.savefig('nyc.png', dpi=DPI, bbox_inches='tight')

##############################################################################
# We can easily retrieve the convex hull of each shape. This corresponds to
# the outer edge of the shapes.
boros.geometry.convex_hull.plot()
plt.xticks(rotation=90)

# Grab the limits which we'll use later
xmin, xmax = plt.gca().get_xlim()
ymin, ymax = plt.gca().get_ylim()

plt.savefig('nyc_hull.png', dpi=DPI, bbox_inches='tight')

##############################################################################
# We'll generate some random dots scattered throughout our data, and will
# use them to perform some set operations with our boroughs. We can use
# GeoPandas to perform unions, intersections, etc.

N = 2000  # number of random points
R = 2000  # radius of buffer in feet

#xmin, xmax, ymin, ymax = 900000, 1080000, 120000, 280000
xc = (xmax - xmin) * np.random.random(N) + xmin
yc = (ymax - ymin) * np.random.random(N) + ymin
pts = GeoSeries([Point(x, y) for x, y in zip(xc, yc)])
mp = pts.buffer(R).unary_union
boros_with_holes = boros.geometry - mp
boros_with_holes.plot()
plt.xticks(rotation=90)
plt.savefig('boros_with_holes.png', dpi=DPI, bbox_inches='tight')

##############################################################################
# Finally, we'll show the holes that were taken out of our boroughs.

holes = boros.geometry & mp
holes.plot()
plt.xticks(rotation=90)
plt.savefig('holes.png', dpi=DPI, bbox_inches='tight')
plt.show()



#!/usr/bin/env/python
"""Installation script

"""

import os
import sys

from setuptools import setup

# ensure the current directory is on sys.path so versioneer can be imported
# when pip uses PEP 517/518 build rules.
# https://github.com/python-versioneer/python-versioneer/issues/193
sys.path.append(os.path.dirname(__file__))

import versioneer  # noqa: E402

LONG_DESCRIPTION = """GeoPandas is a project to add support for geographic data to
`pandas`_ objects.

The goal of GeoPandas is to make working with geospatial data in
python easier. It combines the capabilities of `pandas`_ and `shapely`_,
providing geospatial operations in pandas and a high-level interface
to multiple geometries to shapely. GeoPandas enables you to easily do
operations in python that would otherwise require a spatial database
such as PostGIS.

.. _pandas: http://pandas.pydata.org
.. _shapely: http://shapely.readthedocs.io/en/latest/
"""

if os.environ.get("READTHEDOCS", False) == "True":
    INSTALL_REQUIRES = []
else:
    INSTALL_REQUIRES = [
        "pandas >= 1.0.0",
        "shapely >= 1.7",
        "fiona >= 1.8",
        "pyproj >= 2.6.1.post1",
        "packaging",
    ]

# get all data dirs in the datasets module
data_files = []

for item in os.listdir("geopandas/datasets"):
    if not item.startswith("__"):
        if os.path.isdir(os.path.join("geopandas/datasets/", item)):
            data_files.append(os.path.join("datasets", item, "*"))
        elif item.endswith(".zip"):
            data_files.append(os.path.join("datasets", item))

data_files.append("tests/data/*")


setup(
    name="geopandas",
    version=versioneer.get_version(),
    description="Geographic pandas extensions",
    license="BSD",
    author="GeoPandas contributors",
    author_email="kjordahl@alum.mit.edu",
    url="http://geopandas.org",
    project_urls={
        "Source": "https://github.com/geopandas/geopandas",
    },
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/x-rst",
    packages=[
        "geopandas",
        "geopandas.io",
        "geopandas.tools",
        "geopandas.datasets",
        "geopandas.tests",
        "geopandas.tools.tests",
    ],
    package_data={"geopandas": data_files},
    python_requires=">=3.8",
    install_requires=INSTALL_REQUIRES,
    cmdclass=versioneer.get_cmdclass(),
)



from geopandas import read_file, datasets, clip
from shapely.geometry import box


class Bench:
    def setup(self, *args):
        world = read_file(datasets.get_path("naturalearth_lowres"))
        capitals = read_file(datasets.get_path("naturalearth_cities"))
        self.bounds = [box(*geom.bounds) for geom in world.geometry]
        self.points = capitals

    def time_clip(self):
        for bound in self.bounds:
            clip(self.points, bound)



from geopandas import GeoDataFrame, GeoSeries, read_file, datasets, overlay
import numpy as np
from shapely.geometry import Point, Polygon


class Countries:

    param_names = ['how']
    params = [('intersection', 'union', 'identity', 'symmetric_difference',
               'difference')]

    def setup(self, *args):
        world = read_file(datasets.get_path('naturalearth_lowres'))
        capitals = read_file(datasets.get_path('naturalearth_cities'))
        countries = world[['geometry', 'name']]
        countries = countries.to_crs('+init=epsg:3395')[
            countries.name != "Antarctica"]
        capitals = capitals.to_crs('+init=epsg:3395')
        capitals['geometry'] = capitals.buffer(500000)

        self.countries = countries
        self.capitals = capitals

    def time_overlay(self, how):
        overlay(self.countries, self.capitals, how=how)


class Small:

    param_names = ['how']
    params = [('intersection', 'union', 'identity', 'symmetric_difference',
               'difference')]

    def setup(self, *args):
        polys1 = GeoSeries([Polygon([(0, 0), (2, 0), (2, 2), (0, 2)]),
                            Polygon([(2, 2), (4, 2), (4, 4), (2, 4)])])
        polys2 = GeoSeries([Polygon([(1, 1), (3, 1), (3, 3), (1, 3)]),
                            Polygon([(3, 3), (5, 3), (5, 5), (3, 5)])])

        df1 = GeoDataFrame({'geometry': polys1, 'df1': [1, 2]})
        df2 = GeoDataFrame({'geometry': polys2, 'df2': [1, 2]})

        self.df1, self.df2 = df1, df2

    def time_overlay(self, how):
        overlay(self.df1, self.df2, how=how)


class ManyPoints:

    param_names = ['how']
    params = [('intersection', 'union', 'identity', 'symmetric_difference',
               'difference')]

    def setup(self, *args):

        points = GeoDataFrame(geometry=[Point(i, i) for i in range(1000)])
        base = np.array([[0, 0], [0, 100], [100, 100], [100, 0]])
        polys = GeoDataFrame(
            geometry=[Polygon(base + i * 100) for i in range(10)])

        self.df1, self.df2 = points, polys

    def time_overlay(self, how):
        overlay(self.df1, self.df2, how=how)
